#include "LCDv2.h"

void wait_for_reply() {
	int k,timed_out;
	timed_out = 1; /* default timed_out is true */
	for(k=0;k<=100000;k++)
    	if ( check_for_packet() ) {
    		/* -------
    		 *  DEBUG
    		 * ------- */
			/* ShowReceivedPacket(); */
			timed_out = 0; /* set timed_out to false */
			break;
		}
		
	if(timed_out)
		printf("Timed out waiting for a response.\n");		
}

/* -------------
 *     MAIN
 * ------------- */
int main(int argc, char* argv[]) {
	
	/* Port initialisation */
	if ( Serial_Init(SERIALPORT,BAUDRATE)!=0 ) {
		printf(" [LCD] Could not open port \"%s\" at \"%d\" baud.\n",SERIALPORT,BAUDRATE);
		return(-1);
	} else 
		printf(" [LCD] Running\n");

	signal(SIGINT,signal_handler);
	signal(SIGTERM,signal_handler);

	/* For some reason, Linux seems to buffer up data from the LCD, and they are sometimes
	 * dumped at the start of the program. Clear the serial buffer. */
	while ( BytesAvail() ) GetByte();

	/* Clear LCD */
	outgoing_command.command = 6;
	outgoing_command.data_length = 0;
	send_packet();
	wait_for_reply();
		
	/* Set keypad mask */
	outgoing_command.command = 23;
	outgoing_command.data_length = 2;
	/* Press mask */
	outgoing_command.data[0] = KP_UP | KP_ENTER | KP_CANCEL | KP_LEFT | KP_RIGHT | KP_DOWN;
	/* Release mask */
	outgoing_command.data[1] = 0;
	send_packet();
	wait_for_reply();

	while (1) {
		usleep(100000);  /* 1/10 second */
		
		LcdWriteLine("Hello world !",13,1);
		
		if( check_for_packet() ) { 
			ShowReceivedPacket();
		}
	}

	Uninit_Serial();
	return 0;
}


/* --------------
 * SIGNAL_HANDLER
 * -------------- */
void signal_handler(int sig) {
	printf("  [LCD] Received signal %d. Stop\n",sig);
	/*LcdWriteLine( fd, (unsigned char*)"LCD disabled", 12, 1);*/
	/*LcdWriteLine( fd, (unsigned char*)" ", 1, 2);*/
	Uninit_Serial();
	exit(0);
}

/* ------------
 * LcdWriteLine
 * ------------ */
void LcdWriteLine(char* buffer, int size, int line) {
	int i;
	
    /* Send line to the 633 using command 31 */
	outgoing_command.command = 31;
	outgoing_command.data[0]=0; /*col*/
	outgoing_command.data[1]=line-1; /*row*/

	/* Buffer is completed with spaces */
	memcpy(&outgoing_command.data[2],buffer,size);
	for (i=size;i<16;i++) {
		(&outgoing_command.data[2])[i] = ' ';
	}
	
	outgoing_command.data_length = 18;
	send_packet();

	/* CFA-631 / CFA-633 communications protocol only allows
	 * one outstanding packet at a time. Wait for the response
	 * packet from the CFA-631 / CFA-633 before sending another
	 * packet. */
	wait_for_reply();
}
